var fs = require('fs');
var request = require('request');
var Buffer = require('buffer').Buffer;
var child_process = require('child_process');
var wrench = require('wrench');
var os = require('os');

console.log("############");
console.log("## METEOR ## Preparing meteor for usage.");
console.log("############");
console.log("");

console.log("## Deploying Meteor...");

deployMeteor();

// Non-complaining version of wrench's rmdirSyncRecursive, when stuff is blocking our removal we can't do anything but leave it behind.
var rmdirSyncRecursive = function(path) {
    var files;

    try { files = fs.readdirSync(path); } catch (err) { return; }

    for (var i = 0; i < files.length; i++) {
        var currFile = fs.lstatSync(path + "/" + files[i]);

        if (currFile.isDirectory())
            rmdirSyncRecursive(path + "/" + files[i]);
        else
            try { fs.unlinkSync(path + "/" + files[i]); } catch (err) {}
    }

    try { fs.rmdirSync(path); } catch (err) {}
};

function deployMeteor() {
	// Download Zip from GitHub.
	request('https://github.com/TomWij/meteor-windows/zipball/devel', unpackMeteorZip).pipe(fs.createWriteStream('meteor.zip'));
}

function unpackMeteorZip() {
	// Unpack Zip.
	if (process.platform == "win32")
		request('http://www2.cs.uidaho.edu/~jeffery/win32/unzip.exe', function () {
            setTimeout(function() { // Wait a second to allow any virus scanners to check the file.
    			child_process.exec('unzip meteor.zip -d unpacked', function() {
                    setTimeout(function() { // Wait a second to allow any virus scanners to check the file.        				
        				moveMeteor();
                    }, 1000);
    			});
            }, 1000);
		}).pipe(fs.createWriteStream('unzip.exe'));
	else
		child_process.exec('unzip meteor.zip -d unpacked', moveMeteor);
}

function moveMeteor() {
	// Move Meteor from the unpacked commit folder to the current directory.
	var commitDir = fs.readdirSync('unpacked')[0];
	wrench.copyDirSyncRecursive('unpacked/' + commitDir, '.', { preserve : true }); // Preserve makes sure we don't empty the current directory
	fetchMongo();
}

function fetchMongo() {
    // Fetch and unpack Meteor.
    if (process.platform == "linux") {
        var fileName = 'mongodb-linux-' + os.arch + '-2.0.2';
        request('http://fastdl.mongodb.org/linux/' + fileName + '.tgz', function () {
            setTimeout(function() { // Wait a second to allow any virus scanners to check the file.
                child_process.exec('tar -xz ' + fileName + '.tgz', function() {
                    setTimeout(function() { // Wait a second to allow any virus scanners to check the file.
                        fs.unlinkSync(fileName + '.tgz');
                        fixDevBundle(fileName);
                        outro();
                    }, 1000);
                });
            }, 1000);
        }).pipe(fs.createWriteStream('mongodb-linux-' + os.arch + '-2.0.2.tgz'));
    } else if (process.platform == "darwin") {
        var fileName = 'mongodb-osx-' + os.arch + '-2.0.2';
        request('http://fastdl.mongodb.org/osx/' + filename + '.tgz', function () {
            setTimeout(function() { // Wait a second to allow any virus scanners to check the file.
                child_process.exec('tar -xz ' + filename + '.tgz', function() {
                    setTimeout(function() { // Wait a second to allow any virus scanners to check the file.
                        fs.unlinkSync(filename + '.tgz');
                        fixDevBundle(filename);
                        outro();
                    }, 1000);
                });
            }, 1000);
        }).pipe(fs.createWriteStream(fileName + '.tgz'));
    } else if (process.platform == "win32") {
        var fileName = 'mongodb-win32-x86_64-2008plus-2.0.5';
        request('http://fastdl.mongodb.org/win32/' + fileName + '.zip', function () {
            setTimeout(function() { // Wait a second to allow any virus scanners to check the file.
                child_process.exec('unzip ' + fileName + '.zip', function() {
                    setTimeout(function() { // Wait a second to allow any virus scanners to check the file.
                        fs.unlinkSync(fileName + '.zip');
                        fixDevBundle(fileName);
                        outro();
                    }, 1000);
                });
            }, 1000);
        }).pipe(fs.createWriteStream(fileName + '.zip'));
    } else {
        console.log("");
        console.log("############");
        console.log("## METEOR ## WARNING: Your platform is not supported, you need to install Meteor manually if it supports your platform.");
        console.log("############");
        console.log("");
    }
}

function fixDevBundle(mongoDir) {
	// XXX As this whole installer assumes node.js is already present, we don't put it in the dev bundle.
	//     We could figure out where it is and copy it along, such that the dev bundle can be ported to other computers that don't have Node.js yet.

	fs.mkdirSync('./.git'); // To fool ./meteor into behaving properly. :)
	fs.mkdirSync('./dev_bundle');
	fs.mkdirSync('./dev_bundle/bin');
	if (process.platform == "win32") {
		try {
			wrench.copyDirSyncRecursive(process.env.HOMEDRIVE + '/Program Files/nodejs', './dev_bundle/bin', { preserve : true }); // Preserve makes sure we don't empty the current directory.
		} catch (e) {}
		try {
			wrench.copyDirSyncRecursive(process.env.HOMEDRIVE + '/Program Files (x86)/nodejs', './dev_bundle/bin', { preserve : true }); // Preserve makes sure we don't empty the current directory.
		} catch (e) {}
	} else {
		// XXX TODO: `which node` etc...
	}
    fs.mkdirSync('./dev_bundle/mongodb');
    wrench.copyDirSyncRecursive(mongoDir, './dev_bundle/mongodb', { preserve : true }); // Preserve makes sure we don't empty the current directory.
    rmdirSyncRecursive(mongoDir);
    if (process.platform == "win32") {
        try { fs.unlinkSync('./mongodb/bin/bsondump.exe'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongodump.exe'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongoexport.exe'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongofiles.exe'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongoimport.exe'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongorestore.exe'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongos.exe'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongostat.exe'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongotop.exe'); } catch (err) {}
    } else {
        try { fs.unlinkSync('./mongodb/bin/bsondump'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongodump'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongoexport'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongofiles'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongoimport'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongorestore'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongos'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongosniff'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongostat'); } catch (err) {}
        try { fs.unlinkSync('./mongodb/bin/mongotop'); } catch (err) {}
    }

	var parent = __dirname.replace(/\\/g, '/').replace(/\/+$/, '').replace(/\/[^\/]+\/[^\/]+\/[^\/]+\/?$/, '');
	fs.unlinkSync('meteor.bat');
    fs.writeFileSync('meteor.bat', '@echo off&&set NODE_PATH=' + parent + '/dev_bundle/lib/node_modules&&node "' + parent + '/app/meteor/meteor.js" %*');
    fs.writeFileSync('./dev_bundle/.bundle_version.txt', '0.1.5');
}

function cleanup() {
	fs.unlinkSync('meteor.zip');
    fs.unlinkSync('unzip.exe');
    rmdirSyncRecursive('unpacked');
}

function moveMeteorOutside() {
    wrench.copyDirSyncRecursive('.', '../..', { preserve : true }); // Preserve makes sure we don't empty the current directory.
	fs.mkdirSync('../../dev_bundle/lib');
	fs.mkdirSync('../../dev_bundle/lib/node_modules');
	wrench.copyDirSyncRecursive('../../node_modules', '../../dev_bundle/lib/node_modules', { preserve : true }); // Preserve makes sure we don't empty the current directory.
    rmdirSyncRecursive('../Meteor-Test-Installer');
	rmdirSyncRecursive('../../node_modules');
}

function outro() {
    cleanup();
    moveMeteorOutside();

    if (process.platform == "win32") {
        console.log("")
        console.log("############");
        console.log("## METEOR ## DONE: Add the current folder to your PATH. Then you can use `meteor`, for help type `meteor`, documentation is available at http://docs.meteor.com/");
        console.log("############");
        console.log("");
    }
    else if (process.platform == "linux") {
        //child_process.exec("echo 'export PATH=DIRTOADD:${PATH}' >> ~/.bashrc", function() {
            console.log("");
            console.log("############");
            console.log("## METEOR ## DONE: Add the current folder to your PATH. Then you can use `meteor`, for help type `meteor`, documentation is available at http://docs.meteor.com/");
            console.log("############");
            console.log("");
        //});
    } else if (process.platform == "darwin") {
        console.log("");
		console.log("############");
		console.log("## METEOR ## DONE: Add the current folder to your PATH. Then you can use `meteor`, for help type `meteor`, documentation is available at http://docs.meteor.com/");
		console.log("############");
		console.log("");
    } else {
        console.log("");
        console.log("############");
        console.log("## METEOR ## DONE: Add the current folder to your PATH. Then you can use `meteor`, for help type `meteor`, documentation is available at http://docs.meteor.com/");
        console.log("############");
        console.log("");
    }
}